/*******************************************************************************************************
 * Sprava vlakoveho hlaseni na displeji. Ve smycce se na prvnim radku displeje 16x2 zobrazuje po rade:
 * 1. Aktualni cas ziskany z hostitelskeho PC, 2. Cislo linky a cilova stanice, 3. seznam zastavek na 
 * trase, 4. Pristi zastavka. 5. Stav obslouzeni tlacitka Zastavit.
 * Lukas Fritsch
 * rev. 17.7.11.0
 *******************************************************************************************************/

#ifndef DISPLAY_IN_TRAIN
#define DISPLAY_IN_TRAIN

#include <LiquidCrystal.h>
#include <Arduino.h>
#include <EEPROM.h>

const unsigned long STATION_TYPE_DEFAULT=0; //priznak bezne zastavky
const unsigned long STATION_TYPE_NONE=16777216; //priznak neexistujici zastavky
const unsigned long STATION_TYPE_ON_REQUEST=33554432; //priznak zastavky na znameni

const int MAX_DISPLAYS=6; //pocet ruznych typu informaci na displeji
const int TIME_DISPLAY=0; //typ informace: aktualni cas
const int LINE_DISPLAY=1; //typ informace: cislo linky, napr. U23
const int STATIONS_DISPLAY=2; //typ informace: seznam zastavek na trase
const int NEXT_STATION_DISPLAY=3; //typ informace: pristi zastavka na trase
const int STOP_ON_REQUEST_ACCEPTED=4;//informace na displeji v pripade stisku tlacitka Zastavit
const int MANIPULATION_PURPOSES_DISPLAY=5; //pro ucely prepocitani parametru smycky a pro zobrazeni informace o rozjezdu vlaku

const int COLS_IN_ROW=16; //pocet znaku v radce displeje
const int TIME_TO_VIEW_TEXT=1500; //doba zobrazeni aktualniho casu v milisekundach
const int TIME_TO_WAIT_IN_STATION=7000; //doba setrvani vlaku ve stanici pred dalsi smyckou
const int TIME_BEFORE_PRINT=1400;  //doba od prehrani zvuku do vypisu na displej
const int SCROLL_SPEED=150; //doba zobrazeni textu v milisekundach pred jeho naslednym rolovanim o jednu pozici doleva

const int RX_BUFFER_SZ=64;  //velikost UART rx bufferu
const int GLT_RESPONSE_MIN_LEN=9; //minimalni delka odpovedi na prikaz GLT?
const int NEXT_STATION_MAX_LEN=64;  //maximalni delka retezce nasledujici stanice
const int LINE_CAPTION_MAX_LEN=72;  //maximalni delka retezce linky vlaku (cislo linky + cilova stanice)
const int MAX_STATIONS=30;  //maximalni pocet zastavek na trase
const int STATIONS_MAX_LEN=200;//maximalni delka retezce se seznamem zastavek
const int TRAIN_NO_MAX_LEN=6; //maximalni delka retezce s cislem vlaku

const int STATION_ON_REQUEST=2; //vstup tlacitka pro zastaveni na zastavce na znameni. Stav tlacitka je zdrojem externiho preruseni gpio.
const int STATION_ON_REQUEST_ACCEPTED_LED=3; //LED pro signalizaci, ze na zastavce vlak zastavi

const int EEPROM_DATA_VALID_FLAG=0x09;  //priznak platnosti dat v eeprom
const int EEPROM_DATA_VALID_ADDRESS=0x0000; //adresa v eeprom s priznakem platnosti dat
const int EEPROM_DATA_START_ADDRESS=0x0001; //pocatecni adresa dat linek
  
class CDisplayInTrain
{
private:
  char* strTime;  //ukazatel na retezec s aktualnim casem
  char strTrainNoLineA[TRAIN_NO_MAX_LEN+1]; //cislo linky, napr. T5
  char strTrainNoLineB[TRAIN_NO_MAX_LEN+1]; //cislo linky, napr. T5
  char strLine[LINE_CAPTION_MAX_LEN+1]; //retezec linky vlaku a cilove stanice
  char strDispOneRow[COLS_IN_ROW+1];
  static char strStationsCaption[];
  char strStationsLineA[STATIONS_MAX_LEN];
  //seznam zastavek na trase linky A
  char strStationsLineB[STATIONS_MAX_LEN];
  //seznam zastavek na trase linky B
  static char strNextStationCaption[];
  //titulek pred pristi stanici
  static char strStationOnRequestCaption[];
  //titulek pro zastavku na znameni
  static char strStationOnRequestAcceptedCaption[];
  //titulek pro prijmuty pozadavek na zastaveni
  static char strLeaveTrain[];
  //titulek pro konecnou stanici
  static char strTrainIsStarting[];
  //titulek pro ukonceni vystupu/nastupu
  static char strDoNotOpenDoor[];
  //titulek pro vyckani, az zastavi vlak
  char strNextStation[NEXT_STATION_MAX_LEN+1];
  //retezec pristi stanice
  char strEndStation[NEXT_STATION_MAX_LEN+1];
 //retezec konecne stanice

  int iStrLen;  //pro potreby urceni delky retezce

  int iCurrentDisplay;//aktualni typ informace zobrazeny na displeji
  int iInitialCol;  //sloupec, od nehoz se ma zacit vypisovat text na radek displeje

  unsigned long ListOfStations[MAX_STATIONS]; //seznam zastavek na trase s identifikacnimi udaji jako pocatecni znak zastavky, delka nazvu zastavky
  int iNrOfStations;  //pocet zastavek na trase
  int iNrOfRepeatsOfThisStation;  //pocet opakovani cyklu pro danou stanici. Urcuje "vzdalenost" mezi sousednimi stanicemi (cislo 0-7; 0 = pouze jedno zobrazeni cyklu).
  int iStationId; //aktualni id stanice 0-(iNrOfStations-1)
  int iReadBytes; //pocet prectenych B na UART
  char rxBuffer[RX_BUFFER_SZ+1];  //UART rx buffer
  bool bLineA; //druh linky: true = linka A, false = linka B
  volatile bool bStationOnRequestKeyPressed;  //true, pokud bylo stisknuto tlacitko Zastavit. Nastavuje se v rutine preruseni od tlacitka
  bool bAcceptanceAtLeastOnceWillBeDisplayed;
  bool bIsNotCurrentStationOnRequest;//true, pokud neni stanice na znameni
  bool bEEPROMDataValid;  //true, pokud jsou data ulozena v eeprom platna
  bool bIsInBoot; //true, pokud je uP pripraven na prijem dat linek z hostitelskeho PC
public:
  LiquidCrystal* lcd;
  CDisplayInTrain();
  bool Init();
  void ReadLinesDataFromEEPROM();
  bool WriteLinesDataToEEPROM();
  void GenerateLineCaption(char* _strEndStation);
  bool GetStationName(int _iStationId,char* strDest,int iStrDestMaxLen);
  int CreateListOfStations(char* _strAll);
  void ScrollInText(char * _str, int _iStrLen, int _iRow);
  bool GetLocalTime();
  bool RequestAccepted();
  void AcceptRequest();
  bool StationOnRequest();
  void OnLoop();
  
};

#endif

